<?php
/*
Anti-Spam filtering protection addon for miniBB.
Initial Author: lime (as_ziggy@yahoo.com)
Developing: Paul Puzyrev / miniBB.com
Release date: 2006-10-24
Most recent update: 2023-06-08
Description: Forbid guests and newcomers to post urls, links, email addresses and specified dictionary words; also to prevent massive spam attacks, duplicated messages and topic titles
*/

if (!defined('INCLUDED776')) die ('Fatal error.');

include($pathToFiles.'addon_anti_guest_options.php');

$inAdmins=($logged_admin==1 or $isMod==1);

if(!isset($user_num_posts)) $user_num_posts=0;
$ag_terms_member=(!(isset($ag_membersRuleDictOnly) and $ag_membersRuleDictOnly) and $ag_membersRulePosts>0 and $user_num_posts<=$ag_membersRulePosts);

if($inAdmins or ($user_id>1 and !$ag_terms_member)) {
$anti_guest_terms=array();
}
else $anti_guest_terms=$ag_terms;

if(isset($_POST['topicTitle'])) $topicTitle=$_POST['topicTitle']; else $topicTitle='';
if(isset($_POST['postText'])) $postText=$_POST['postText']; else $postText='';
$checkedText=operate_string($topicTitle).operate_string($postText);
	
//fix to not include the original forum or website URL
$checkedText=preg_replace(array("#http[s]*://".$main_url."#is", "#\[url=".$main_url."#is", "#\[urlc=".$main_url."#is", "#\[img\]".$main_url."#is", "#\[imgs\]".$main_url."#is", "#\[imgs=".$main_url."#is", "#\[img=".$main_url."#is", "#".$main_url."#is"), '', $checkedText);

if(isset($site_url)) $checkedText=preg_replace(array("#http[s]*://".$site_url."#is", "#\[url=".$site_url."#is", "#\[urlc=".$site_url."#is", "#\[img\]".$site_url."#is", "#\[imgs\]".$site_url."#is", "#\[imgs=".$site_url."#is", "#\[img=".$site_url."#is", "#".$site_url."#is"), '', $checkedText);

$guest_offence=0;

//check for $ag_terms if required
if(sizeof($anti_guest_terms)>0){
foreach($anti_guest_terms as $v){
if(substr_count(strtolower_unicode($checkedText), strtolower_unicode($v))>0){
$guest_offence=1;
break;
}
}
}

//check for dictionary, if no URLs found
if(!$inAdmins and $guest_offence==0){
include($pathToFiles.'addon_anti_guest_dict.php');
foreach($forbidden_text as $v){
if(substr_count(strtolower_unicode($checkedText), strtolower_unicode($v))>0){
$guest_offence=2;
$found_word=$v;
break;
}
}
}

//check for 100% similar message or topic title
if(!$inAdmins and $guest_offence==0){

require_once($pathToFiles.'bb_func_txt.php');

$TT='';
if(isset($_POST['topicTitle'])) {
$TT=trim($_POST['topicTitle']);
if(strlen($TT)>0 and strlen_unicode($TT)<$post_text_minlength) $TT='';
$TT=str_replace(array('&#032;', '&#32;'), '', $TT);
$topicTitle=textFilter($TT,$topic_max_length,$post_word_maxlength,0,1,0,$user_id,255);
}
else $topicTitle='';

if(!isset($_POST['postText']) or trim($_POST['postText'])=='') $postText=$TT; else $postText=trim($_POST['postText']);
$postText=str_replace(array('&#032;', '&#32;'), '', $postText);

if(!($logged_admin==1 or $isMod==1) and (strlen_unicode($postText)>10 or strlen_unicode($topicTitle)>10)){

if(isset($clForumsUsers)) $closedForums=getAccess($clForums, $clForumsUsers, $user_id); else $closedForums='n';
if($closedForums!='n') $xtr=getClForums($closedForums,'and','','forum_id','and','!='); else $xtr='';

if(!isset($_POST['disbbcode']) or (isset($_POST['disbbcode']) and $_POST['disbbcode']=='') ) $disbbcode=FALSE; else $disbbcode=TRUE;

$postText=textFilter($postText,$post_text_maxlength,$post_word_maxlength,1,$disbbcode,1,$user_id);

if($action!='editmsg2' and $row=db_simpleSelect(0, $Tp, 'post_id, forum_id, topic_id', 'post_text', '=', $postText, 'post_id desc', 1)){
list($existing_msg_id, $existing_forum_id, $existing_topic_id)=$row;
$guest_offence=3;
}

if($guest_offence==0 and $topicTitle!=''){
//topic title check
if($row=db_simpleSelect(0, $Tt, 'forum_id, topic_id, topic_title', 'topic_title', '=', $topicTitle, 'topic_id desc', 1)){
list($forum_id, $topic_id, $topicTitle)=$row;
if($row=db_simpleSelect(0, $Tf, 'forum_id, forum_name', 'forum_id', '=', $forum_id)) $forumName=$row[0]; else $forumName='N/A';
if(isset($mod_rewrite) and $mod_rewrite) {
$topic_url=genTopicURL($main_url, $forum, $forumName, $topic, $topicTitle);
}
else {
$topic_url="{$main_url}/{$indexphp}action=vthread&amp;forum=$forum_id&amp;topic=$topic_id";
}
$guest_offence=4;
}
}

}
}

//Check for post limits if enabled
if(!$inAdmins and $guest_offence==0 and $ag_limitsEnabled and $user_num_posts<=$ag_limitsActive and !in_array($user_id, $ag_limitsExceptMemberIDs)){

$timeChk=date('Y-m-d H:i:s', time()-86400);

//how many posts and topics were done in 24 hours - compare by IP first, then by user ID; choose the biggest

if($row=db_simpleSelect(0, $Tp, 'count(*)', 'poster_ip', '=', $thisIp, '', '', 'post_time', '>=', $timeChk)) $currPostsIp=$row[0]; else $currPostsIp=0;

$tReq="select count(*) from {$Tt}, {$Tp} where {$Tp}.poster_ip='{$thisIp}' and {$Tp}.post_time>='{$timeChk}' and {$Tp}.topic_id={$Tt}.topic_id group by {$Tt}.topic_id";

if($res=mysqli_query($mysqlink, $tReq) and $resNum=mysqli_num_rows($res)) $currTopicsIp=$resNum; else $currTopicsIp=0;

//echo 'Only IP posts: '.$currPostsIp.' -- ';

$currPostsIp=$currPostsIp-$currTopicsIp;

//echo 'IP Posts without topics: '.$currPostsIp.' -- IP Topics: '.$currTopicsIp.'<br>';

/**/

if($user_id>0) {

if($row=db_simpleSelect(0, $Tp, 'count(*)', 'poster_id', '=', $user_id, '', '', 'post_time', '>=', $timeChk)) $currPostsId=$row[0]; else $currPostsId=0;

if($row=db_simpleSelect(0, $Tt, 'count(*)', 'topic_poster', '=', $user_id, '', '', 'topic_time', '>=', $timeChk)) $currTopicsId=$row[0]; else $currTopicsId=0;

//echo 'Only ID posts: '.$currPostsId;

$currPostsId=$currPostsId-$currTopicsId;

}
else{
$currPostsId=0; $currTopicsId=0;
}

$currPostsNum=max($currPostsIp, $currPostsId);
$currTopicsNum=max($currTopicsIp, $currTopicsId);

//echo ' -- ID Posts without topics: '.$currPostsId.' -- ID Topics: '.$currTopicsId.'<br>';
//echo 'Final posts: '.$currPostsNum.' -- Final topics: '.$currTopicsNum.'<br>';


//which limitation to choose
if($user_num_posts<=$ag_limitsInitial) {
$limPosts=$ag_limitsInitialPosts; $limTopics=$ag_limitsInitialTopics;
}
else{
$limPosts=$ag_limitsActivePosts; $limTopics=$ag_limitsActiveTopics;
}

//echo 'Lim posts: '.$limPosts.' -- Lim topics: '.$limTopics.'<br>';

if($action=='pthread' and $currPostsNum>$limPosts) $guest_offence=5;
elseif($action=='ptopic' and $currTopicsNum>$limTopics) $guest_offence=6;

}

//If a forbidden post is detected, show warning page
//echo $action.'::'.$guest_offence;
//exit;
unset($xtr);

if($guest_offence>0){

$errorMSG=$anti_guest_title;
$title.=$anti_guest_title;

$displayFormElements=array('topicTitle'=>1, 'postText'=>1);

if($guest_offence==1) $antiWarn=$anti_guest_msg;
elseif($guest_offence==2) $antiWarn=str_replace('{FORBIDDEN_WORD}', $found_word, $anti_guest_dict);
elseif($guest_offence==3) $antiWarn=str_replace(array('{EXISTING_MSG_ID}', '{EXISTING_FORUM_ID}', '{EXISTING_TOPIC_ID}'), array($existing_msg_id, $existing_forum_id, $existing_topic_id), $anti_guest_similar);
elseif($guest_offence==4) $antiWarn=str_replace('{TOPIC_URL}', $topic_url, $anti_guest_similar_topic);
elseif($guest_offence==5) $antiWarn=str_replace('{POSTS_MAX}', $limPosts, $anti_guest_posts_limit);
elseif($guest_offence==6) $antiWarn=str_replace('{TOPICS_MAX}', $limTopics, $anti_guest_topics_limit);

if($guest_offence<3){
if($ag_membersRulePosts>0) $antiWarn.=str_replace('{POSTS_REQUIRED}', $ag_membersRulePosts, $anti_guest_rule1);
else $antiWarn.=$anti_guest_rule2;
}

if(isset($_POST['prevForm']) and (integer)$_POST['prevForm']+0==1){
unset($_POST['topicTitle']);
$_POST['postText']=$antiWarn;
$_POST['html']='1';
//exclude condition for preview
}

else{
$l_postholdRepeat=$l_editPost;

if($guest_offence==4 or $guest_offence==5) $fieldsReadOnly=TRUE;
include($pathToFiles.'bb_func_posthold.php');

//Loading header & warning page
echo load_header();
echo ParseTpl(makeUp('main_posthold'));
display_footer();
exit;
}
}
?>